@@ -1,7 +1,7 @@
 /*  bam2depth.c -- depth subcommand.
 
     Copyright (C) 2011, 2012 Broad Institute.
-    Copyright (C) 2012-2016, 2018, 2019-2021 Genome Research Ltd.
+    Copyright (C) 2012-2016, 2018, 2019-2022, 2024 Genome Research Ltd.
 
     Author: Heng Li <lh3@sanger.ac.uk> (to 2020)
     Author: James Bonfield <jkb@sanger.ac.uk> (2021 rewrite)
@@ -72,6 +72,8 @@ typedef struct {
 typedef struct {
     int header;
     int flag;
+    int incl_flag;
+    int require_flag;
     int min_qual;
     int min_mqual;
     int min_len;
@@ -564,7 +566,11 @@ static int fastdepth_core(depth_opt *opt, uint32_t nfiles, char **fn,
             if (b[i]->core.tid < 0)
                 continue;
             if (b[i]->core.flag & opt->flag)
-                continue;
+                continue; // must have none of the flags set
+            if (opt->incl_flag && (b[i]->core.flag & opt->incl_flag) == 0)
+                continue; // must have at least one flag set
+            if ((b[i]->core.flag & opt->require_flag) != opt->require_flag)
+                continue; // must have all lags set
             if (b[i]->core.qual < opt->min_mqual)
                 continue;
 
@@ -654,7 +660,11 @@ static int fastdepth_core(depth_opt *opt, uint32_t nfiles, char **fn,
             if (b[i]->core.tid < 0)
                 continue;
             if (b[i]->core.flag & opt->flag)
-                continue;
+                continue; // must have none of the flags set
+            if (opt->incl_flag && (b[i]->core.flag & opt->incl_flag) == 0)
+                continue; // must have at least one flag set
+            if ((b[i]->core.flag & opt->require_flag) != opt->require_flag)
+                continue; // must have all lags set
             if (b[i]->core.qual < opt->min_mqual)
                 continue;
 
@@ -712,8 +722,14 @@ static void usage_exit(FILE *fp, int exit_status)
     fprintf(fp, "  -b FILE      Use bed FILE for list of regions\n");
     fprintf(fp, "  -f FILE      Specify list of input BAM/SAM/CRAM filenames\n");
     fprintf(fp, "  -X           Use custom index files (in -X *.bam *.bam.bai order)\n");
-    fprintf(fp, "  -g INT       Remove specified flags from default flag filter\n");
-    fprintf(fp, "  -G INT       Add specified flags to the default flag filter\n");
+    fprintf(fp, "  -g INT       Remove specified flags from default filter-out flag list\n");
+    fprintf(fp, "  -G, --excl-flags FLAGS\n");
+    fprintf(fp, "               Add specified flags to the  default filter-out flag list\n");
+    fprintf(fp, "               [UNMAP,SECONDARY,QCFAIL,DUP]\n");
+    fprintf(fp, "      --incl-flags FLAGS\n");
+    fprintf(fp, "               Only include records with at least one the FLAGs present [0]\n");
+    fprintf(fp, "      --require-flags FLAGS\n");
+    fprintf(fp, "               Only include records with all of the FLAGs present [0]\n");
     fprintf(fp, "  -H           Print a file header line\n");
     fprintf(fp, "  -l INT       Minimum read length [0]\n");
     fprintf(fp, "  -o FILE      Write output to FILE [stdout]\n");
@@ -721,16 +737,15 @@ static void usage_exit(FILE *fp, int exit_status)
                 "               Filter bases with base quality smaller than INT [0]\n");
     fprintf(fp, "  -Q, --min-MQ INT\n"
                 "               Filter alignments with mapping quality smaller than INT [0]\n");
-    fprintf(fp, "  -H           Print a file header\n");
     fprintf(fp, "  -J           Include reads with deletions in depth computation\n");
     fprintf(fp, "  -s           Do not count overlapping reads within a template\n");
-    sam_global_opt_help(fp, "-.---@-.");
+    sam_global_opt_help(fp, "-.--.@-.");
     exit(exit_status);
 }
 
 int main_depth(int argc, char *argv[])
 {
-    int nfiles, i;
+    int nfiles, i, tmp_flag;
     samFile **fp;
     sam_hdr_t **header;
     int c, has_index_file = 0;
@@ -738,6 +753,8 @@ int main_depth(int argc, char *argv[])
     char *out_file = NULL;
     depth_opt opt = {
         .flag = BAM_FUNMAP | BAM_FSECONDARY | BAM_FDUP | BAM_FQCFAIL,
+        .incl_flag = 0,
+        .require_flag = 0,
         .min_qual = 0,
         .min_mqual = 0,
         .skip_del = 1,
@@ -752,11 +769,14 @@ int main_depth(int argc, char *argv[])
 
     sam_global_args ga = SAM_GLOBAL_ARGS_INIT;
     static const struct option lopts[] = {
-        {"min-MQ", required_argument, NULL, 'Q'},
-        {"min-mq", required_argument, NULL, 'Q'},
-        {"min-BQ", required_argument, NULL, 'q'},
-        {"min-bq", required_argument, NULL, 'q'},
-        SAM_OPT_GLOBAL_OPTIONS('-', 0, '-', '-', '-', '@'),
+        {"min-MQ",        required_argument, NULL, 'Q'},
+        {"min-mq",        required_argument, NULL, 'Q'},
+        {"min-BQ",        required_argument, NULL, 'q'},
+        {"min-bq",        required_argument, NULL, 'q'},
+        {"excl-flags",    required_argument, NULL, 'G'},
+        {"incl-flags",    required_argument, NULL, 1},
+        {"require-flags", required_argument, NULL, 2},
+        SAM_OPT_GLOBAL_OPTIONS('-', 0, '-', '-', 0, '@'),
         {NULL, 0, NULL, 0}
     };
 
@@ -786,10 +806,44 @@ int main_depth(int argc, char *argv[])
             break;
 
         case 'g':
-            opt.flag &= ~bam_str2flag(optarg);
+            tmp_flag = bam_str2flag(optarg);
+
+            if (tmp_flag < 0) {
+                print_error("depth", "Unknown flag '%s'", optarg);
+                return 1;
+            }
+
+            opt.flag &= ~tmp_flag;
             break;
-        case 'G':
-            opt.flag |= bam_str2flag(optarg);
+        case 'G': // reject if any set
+            tmp_flag = bam_str2flag(optarg);
+
+            if (tmp_flag < 0) {
+                print_error("depth", "Unknown flag '%s'", optarg);
+                return 1;
+            }
+
+            opt.flag |= tmp_flag;
+            break;
+        case 1: // reject unless at least one set (0 means ignore option)
+            tmp_flag = bam_str2flag(optarg);
+
+            if (tmp_flag < 0) {
+                print_error("depth", "Unknown flag '%s'", optarg);
+                return 1;
+            }
+
+            opt.incl_flag |= tmp_flag;
+            break;
+        case 2: // reject unless all set
+            tmp_flag = bam_str2flag(optarg);
+
+            if (tmp_flag < 0) {
+                print_error("depth", "Unknown flag '%s'", optarg);
+                return 1;
+            }
+
+            opt.require_flag |= tmp_flag;
             break;
 
         case 'l':
